@@ -1,14 +1,15 @@
-from typing import Any, Tuple
+from typing import Any
 
-from pandas.core.series import Series
+from pandas import Series
 
-from ..caller_base import CallerBase
 from ..error.illegal_attr_checker import IllegalAttrChecker
 from .graph_object import Graph
+from .graph_type_check import from_graph_type_check
+from graphdatascience.graph.graph_create_result import GraphCreateResult
 
 
-class GraphProjectRunner(CallerBase, IllegalAttrChecker):
-    def __call__(self, graph_name: str, node_spec: Any, relationship_spec: Any, **config: Any) -> Tuple[Graph, Series]:
+class GraphProjectRunner(IllegalAttrChecker):
+    def __call__(self, graph_name: str, node_spec: Any, relationship_spec: Any, **config: Any) -> GraphCreateResult:
         result = self._query_runner.run_query_with_logging(
             f"CALL {self._namespace}($graph_name, $node_spec, $relationship_spec, $config)",
             {
@@ -19,15 +20,15 @@ class GraphProjectRunner(CallerBase, IllegalAttrChecker):
             },
         ).squeeze()
 
-        return Graph(graph_name, self._query_runner, self._server_version), result
+        return GraphCreateResult(Graph(graph_name, self._query_runner, self._server_version), result)
 
-    def estimate(self, node_spec: Any, relationship_spec: Any, **config: Any) -> Series:
+    def estimate(self, node_projection: Any, relationship_projection: Any, **config: Any) -> "Series[Any]":
         self._namespace += ".estimate"
         result = self._query_runner.run_query(
             f"CALL {self._namespace}($node_spec, $relationship_spec, $config)",
             {
-                "node_spec": node_spec,
-                "relationship_spec": relationship_spec,
+                "node_spec": node_projection,
+                "relationship_spec": relationship_projection,
                 "config": config,
             },
         )
@@ -38,6 +39,9 @@ class GraphProjectRunner(CallerBase, IllegalAttrChecker):
     def cypher(self) -> "GraphProjectRunner":
         return GraphProjectRunner(self._query_runner, self._namespace + ".cypher", self._server_version)
 
+
+class GraphProjectBetaRunner(IllegalAttrChecker):
+    @from_graph_type_check
     def subgraph(
         self,
         graph_name: str,
@@ -45,7 +49,7 @@ class GraphProjectRunner(CallerBase, IllegalAttrChecker):
         node_filter: str,
         relationship_filter: str,
         **config: Any,
-    ) -> Tuple[Graph, Series]:
+    ) -> GraphCreateResult:
         self._namespace += ".subgraph"
         result = self._query_runner.run_query_with_logging(
             f"CALL {self._namespace}($graph_name, $from_graph_name, $node_filter, $relationship_filter, $config)",
@@ -58,4 +62,4 @@ class GraphProjectRunner(CallerBase, IllegalAttrChecker):
             },
         ).squeeze()
 
-        return Graph(graph_name, self._query_runner, self._server_version), result
+        return GraphCreateResult(Graph(graph_name, self._query_runner, self._server_version), result)
