@@ -2,13 +2,12 @@ from typing import Any, Tuple
 
 from pandas import Series
 
-from ..caller_base import CallerBase
 from ..error.illegal_attr_checker import IllegalAttrChecker
 from ..error.uncallable_namespace import UncallableNamespace
 from .nr_training_pipeline import NRTrainingPipeline
 
 
-class NRPipelineCreateRunner(CallerBase, UncallableNamespace, IllegalAttrChecker):
+class NRPipelineCreateRunner(UncallableNamespace, IllegalAttrChecker):
     def create(self, name: str) -> Tuple[NRTrainingPipeline, "Series[Any]"]:
         self._namespace += ".create"
 
