@@ -2,33 +2,30 @@ from typing import Any, Optional
 
 from pandas import DataFrame, Series
 
-from ..caller_base import CallerBase
 from ..error.client_only_endpoint import client_only_endpoint
 from ..error.illegal_attr_checker import IllegalAttrChecker
 from ..error.uncallable_namespace import UncallableNamespace
-from .lp_pipeline_create_runner import LPPipelineCreateRunner
+from ..model.pipeline_model import PipelineModel
 from .lp_training_pipeline import LPTrainingPipeline
-from .nc_pipeline_create_runner import NCPipelineCreateRunner
 from .nc_training_pipeline import NCTrainingPipeline
-from .nr_pipeline_create_runner import NRPipelineCreateRunner
 from .nr_training_pipeline import NRTrainingPipeline
 from .training_pipeline import TrainingPipeline
+from graphdatascience.server_version.server_version import ServerVersion
 
 
-class PipelineProcRunner(CallerBase, UncallableNamespace, IllegalAttrChecker):
-    @property
-    def linkPrediction(self) -> LPPipelineCreateRunner:
-        return LPPipelineCreateRunner(self._query_runner, f"{self._namespace}.linkPrediction", self._server_version)
+class PipelineProcRunner(UncallableNamespace, IllegalAttrChecker):
+    @client_only_endpoint("gds.pipeline")
+    def get(self, pipeline_name: str) -> TrainingPipeline[PipelineModel]:
+        # as it was a client only endpoint, exists need
+        if self._server_version < ServerVersion(2, 5, 0):
+            self._namespace = "gds.beta.pipeline"
+        result = self.exists(pipeline_name)
+        if result["exists"]:
+            return self._resolve_pipeline(result["pipelineType"], result["pipelineName"])
+        else:
+            raise ValueError(f"No pipeline named '{pipeline_name}' exists")
 
-    @property
-    def nodeClassification(self) -> NCPipelineCreateRunner:
-        return NCPipelineCreateRunner(self._query_runner, f"{self._namespace}.nodeClassification", self._server_version)
-
-    @property
-    def nodeRegression(self) -> NRPipelineCreateRunner:
-        return NRPipelineCreateRunner(self._query_runner, f"{self._namespace}.nodeRegression", self._server_version)
-
-    def list(self, pipeline: Optional[TrainingPipeline] = None) -> DataFrame:
+    def list(self, pipeline: Optional[TrainingPipeline[PipelineModel]] = None) -> DataFrame:
         self._namespace += ".list"
 
         if pipeline:
@@ -48,7 +45,7 @@ class PipelineProcRunner(CallerBase, UncallableNamespace, IllegalAttrChecker):
 
         return self._query_runner.run_query(query, params).squeeze()  # type: ignore
 
-    def drop(self, pipeline: TrainingPipeline) -> "Series[Any]":
+    def drop(self, pipeline: TrainingPipeline[PipelineModel]) -> "Series[Any]":
         self._namespace += ".drop"
 
         query = f"CALL {self._namespace}($pipeline_name)"
@@ -56,19 +53,7 @@ class PipelineProcRunner(CallerBase, UncallableNamespace, IllegalAttrChecker):
 
         return self._query_runner.run_query(query, params).squeeze()  # type: ignore
 
-    @client_only_endpoint("gds.pipeline")
-    def get(self, pipeline_name: str) -> TrainingPipeline:
-        query = "CALL gds.beta.pipeline.list($pipeline_name)"
-        params = {"pipeline_name": pipeline_name}
-        result = self._query_runner.run_query(query, params)
-
-        if len(result) == 0:
-            raise ValueError(f"No pipeline named '{pipeline_name}' exists")
-
-        pipeline_type = result["pipelineType"][0]
-        return self._resolve_pipeline(pipeline_type, pipeline_name)
-
-    def _resolve_pipeline(self, pipeline_type: str, pipeline_name: str) -> TrainingPipeline:
+    def _resolve_pipeline(self, pipeline_type: str, pipeline_name: str) -> TrainingPipeline[PipelineModel]:
         if pipeline_type == "Node classification training pipeline":
             return NCTrainingPipeline(pipeline_name, self._query_runner, self._server_version)
         elif pipeline_type == "Link prediction training pipeline":
