@@ -1,19 +1,20 @@
 import base64
 import json
 import time
-from typing import Any, Dict, Optional, Tuple
+import warnings
+from typing import Any, Dict, List, Optional, Tuple
 
 import pyarrow.flight as flight
 from pandas import DataFrame
 from pyarrow.flight import ClientMiddleware, ClientMiddlewareFactory
 
+from ..server_version.server_version import ServerVersion
 from .arrow_graph_constructor import ArrowGraphConstructor
 from .graph_constructor import GraphConstructor
 from .query_runner import QueryRunner
 from graphdatascience.server_version.compatible_with import (
     IncompatibleServerVersionError,
 )
-from graphdatascience.server_version.server_version import ServerVersion
 
 
 class ArrowQueryRunner(QueryRunner):
@@ -46,16 +47,26 @@ class ArrowQueryRunner(QueryRunner):
 
         self._flight_client = flight.FlightClient(location, **client_options)
 
+    def warn_about_deprecation(self, old_endpoint: str, new_endpoint: str) -> None:
+        warnings.warn(
+            DeprecationWarning(f"The endpoint '{old_endpoint}' is deprecated. Please use '{new_endpoint}' instead.")
+        )
+
     def run_query(
-        self, query: str, params: Optional[Dict[str, Any]] = None, database: Optional[str] = None
+        self,
+        query: str,
+        params: Optional[Dict[str, Any]] = None,
+        database: Optional[str] = None,
+        custom_error: bool = True,
     ) -> DataFrame:
         if params is None:
             params = {}
 
         new_endpoint_server_version = ServerVersion(2, 2, 0)
+        no_tier_in_namespace_server_version = ServerVersion(2, 5, 0)
 
         # We need to support the deprecated endpoints until they get removed on the server side
-        if "gds.graph.streamNodeProperty" in query or "gds.graph.nodeProperty.stream" in query:
+        if (old_endpoint := ("gds.graph.streamNodeProperty" in query)) or "gds.graph.nodeProperty.stream" in query:
             graph_name = params["graph_name"]
             property_name = params["properties"]
             node_labels = params["entities"]
@@ -64,11 +75,17 @@ class ArrowQueryRunner(QueryRunner):
                 endpoint = "gds.graph.streamNodeProperty"
             else:
                 endpoint = "gds.graph.nodeProperty.stream"
+                if old_endpoint:
+                    self.warn_about_deprecation(
+                        old_endpoint="gds.graph.streamNodeProperty", new_endpoint="gds.graph.nodeProperty.stream"
+                    )
 
             return self._run_arrow_property_get(
                 graph_name, endpoint, {"node_property": property_name, "node_labels": node_labels}
             )
-        elif "gds.graph.streamNodeProperties" in query or "gds.graph.nodeProperties.stream" in query:
+        elif (
+            old_endpoint := ("gds.graph.streamNodeProperties" in query)
+        ) or "gds.graph.nodeProperties.stream" in query:
             graph_name = params["graph_name"]
             property_names = params["properties"]
             node_labels = params["entities"]
@@ -77,12 +94,18 @@ class ArrowQueryRunner(QueryRunner):
                 endpoint = "gds.graph.streamNodeProperties"
             else:
                 endpoint = "gds.graph.nodeProperties.stream"
+                if old_endpoint:
+                    self.warn_about_deprecation(
+                        old_endpoint="gds.graph.streamNodeProperties", new_endpoint="gds.graph.nodeProperties.stream"
+                    )
             return self._run_arrow_property_get(
                 graph_name,
                 endpoint,
                 {"node_properties": property_names, "node_labels": node_labels},
             )
-        elif "gds.graph.streamRelationshipProperty" in query or "gds.graph.relationshipProperty.stream" in query:
+        elif (
+            old_endpoint := ("gds.graph.streamRelationshipProperty" in query)
+        ) or "gds.graph.relationshipProperty.stream" in query:
             graph_name = params["graph_name"]
             property_name = params["properties"]
             relationship_types = params["entities"]
@@ -91,13 +114,19 @@ class ArrowQueryRunner(QueryRunner):
                 endpoint = "gds.graph.streamRelationshipProperty"
             else:
                 endpoint = "gds.graph.relationshipProperty.stream"
-
+                if old_endpoint:
+                    self.warn_about_deprecation(
+                        old_endpoint="gds.graph.streamRelationshipProperty",
+                        new_endpoint="gds.graph.relationshipProperty.stream",
+                    )
             return self._run_arrow_property_get(
                 graph_name,
                 endpoint,
                 {"relationship_property": property_name, "relationship_types": relationship_types},
             )
-        elif "gds.graph.streamRelationshipProperties" in query or "gds.graph.relationshipProperties.stream" in query:
+        elif (
+            old_endpoint := ("gds.graph.streamRelationshipProperties" in query)
+        ) or "gds.graph.relationshipProperties.stream" in query:
             graph_name = params["graph_name"]
             property_names = params["properties"]
             relationship_types = params["entities"]
@@ -106,13 +135,20 @@ class ArrowQueryRunner(QueryRunner):
                 endpoint = "gds.graph.streamRelationshipProperties"
             else:
                 endpoint = "gds.graph.relationshipProperties.stream"
+                if old_endpoint:
+                    self.warn_about_deprecation(
+                        old_endpoint="gds.graph.streamRelationshipProperties",
+                        new_endpoint="gds.graph.relationshipProperties.stream",
+                    )
 
             return self._run_arrow_property_get(
                 graph_name,
                 endpoint,
                 {"relationship_properties": property_names, "relationship_types": relationship_types},
             )
-        elif "gds.beta.graph.relationships.stream" in query:
+        elif (
+            old_endpoint := ("gds.beta.graph.relationships.stream" in query)
+        ) or "gds.graph.relationships.stream" in query:
             graph_name = params["graph_name"]
             relationship_types = params["relationship_types"]
 
@@ -122,11 +158,19 @@ class ArrowQueryRunner(QueryRunner):
                     f"server version >= 2.2.0. The current version is {self._server_version}"
                 )
             else:
-                endpoint = "gds.beta.graph.relationships.stream"
+                if self._server_version < no_tier_in_namespace_server_version:
+                    endpoint = "gds.beta.graph.relationships.stream"
+                else:
+                    endpoint = "gds.graph.relationships.stream"
+                    if old_endpoint:
+                        self.warn_about_deprecation(
+                            old_endpoint="gds.beta.graph.relationships.stream",
+                            new_endpoint="gds.graph.relationships.stream",
+                        )
 
             return self._run_arrow_property_get(graph_name, endpoint, {"relationship_types": relationship_types})
 
-        return self._fallback_query_runner.run_query(query, params, database)
+        return self._fallback_query_runner.run_query(query, params, database, custom_error)
 
     def run_query_with_logging(
         self, query: str, params: Optional[Dict[str, Any]] = None, database: Optional[str] = None
@@ -137,11 +181,23 @@ class ArrowQueryRunner(QueryRunner):
     def set_database(self, database: str) -> None:
         self._fallback_query_runner.set_database(database)
 
+    def set_bookmarks(self, bookmarks: Optional[Any]) -> None:
+        self._fallback_query_runner.set_bookmarks(bookmarks)
+
     def database(self) -> Optional[str]:
         return self._fallback_query_runner.database()
 
+    def bookmarks(self) -> Optional[Any]:
+        return self._fallback_query_runner.bookmarks()
+
+    def last_bookmarks(self) -> Optional[Any]:
+        return self._fallback_query_runner.last_bookmarks()
+
     def close(self) -> None:
         self._fallback_query_runner.close()
+        # PyArrow 7 did not expose a close method yet
+        if hasattr(self._flight_client, "close"):
+            self._flight_client.close()
 
     def fallback_query_runner(self) -> QueryRunner:
         return self._fallback_query_runner
@@ -166,7 +222,9 @@ class ArrowQueryRunner(QueryRunner):
 
         return result
 
-    def create_graph_constructor(self, graph_name: str, concurrency: int) -> GraphConstructor:
+    def create_graph_constructor(
+        self, graph_name: str, concurrency: int, undirected_relationship_types: Optional[List[str]]
+    ) -> GraphConstructor:
         database = self.database()
         if not database:
             raise ValueError(
@@ -174,7 +232,9 @@ class ArrowQueryRunner(QueryRunner):
                 "using `GraphDataScience.set_database`."
             )
 
-        return ArrowGraphConstructor(database, graph_name, self._flight_client, concurrency)
+        return ArrowGraphConstructor(
+            database, graph_name, self._flight_client, concurrency, undirected_relationship_types
+        )
 
 
 class AuthFactory(ClientMiddlewareFactory):  # type: ignore
