@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from typing import Any, Dict, Optional
+from typing import Any, Dict, List, Optional
 
 from pandas import DataFrame
 
@@ -10,29 +10,47 @@ from .graph_constructor import GraphConstructor
 class QueryRunner(ABC):
     @abstractmethod
     def run_query(
-        self, query: str, params: Optional[Dict[str, Any]] = None, database: Optional[str] = None
+        self,
+        query: str,
+        params: Optional[Dict[str, Any]] = None,
+        database: Optional[str] = None,
+        custom_error: bool = True,
     ) -> DataFrame:
         pass
 
     def run_query_with_logging(
         self, query: str, params: Optional[Dict[str, Any]] = None, database: Optional[str] = None
     ) -> DataFrame:
-        return self.run_query(query, params, database)
+        return self.run_query(query, params, database, True)
 
     @abstractmethod
     def set_database(self, database: str) -> None:
         pass
 
+    @abstractmethod
+    def set_bookmarks(self, bookmarks: Optional[Any]) -> None:
+        pass
+
     def close(self) -> None:
         pass
 
     @abstractmethod
-    def create_graph_constructor(self, graph_name: str, concurrency: int) -> GraphConstructor:
+    def create_graph_constructor(
+        self, graph_name: str, concurrency: int, undirected_relationship_types: Optional[List[str]]
+    ) -> GraphConstructor:
         pass
 
     @abstractmethod
     def database(self) -> Optional[str]:
         pass
 
+    @abstractmethod
+    def bookmarks(self) -> Optional[Any]:
+        pass
+
+    @abstractmethod
+    def last_bookmarks(self) -> Optional[Any]:
+        pass
+
     def set_server_version(self, _: ServerVersion) -> None:
         pass
